/*
** Module   : WidgetFactory.java
** Abstract :
**
** Copyright (c) 2011-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SIY 20111221 Created initial version
** 002 MAG 20140410 Add RECTANGLE widget creation.
** 003 EVL 20140610 Added TOGGLE-BOX widget support.
** 004 GES 20140625 Minor cleanup.
** 005 CA  20140711 Added MESSAGE-AREA widget support.
** 006 CA  20140805 Added generics to force the abstract widget classes to not use implementation-
**                  specific code: the abstract widgets will have access only to the base APIs
**                  provided by the OutputManager.
**                  Added support for IMAGE widget creation.
** 007 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 008 MAG 20141009 Deleted createMessageLine.
** 009 EVL 20141118 Adding scroll container creation method.
** 010 CA  20150323 Pass the frame ID when creating a label (to properly resolve the font in GUI).
** 011 EVL 20150428 Adding drop-down container creation.
*/

package com.goldencode.p2j.ui.client.driver;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * This interface provides access to driver-specific widget factory necessary to 
 * hide from client-level code implementations of driver-specific widgets.
 */
public interface WidgetFactory<O extends OutputManager<?>>
{
   /**
    * Create new {@link Label} instance for given text.
    * 
    * @param   id
    *          The widget's ID.
    * @param   text
    *          Label text. 
    * @param   frameId
    *          The ID of the frame to which the label belongs, or -1 if unknown at the time of the 
    *          creation.
    * 
    * @return  new {@link Label} instance.
    */
   Label<O> createLabel(WidgetId id, String text, int frameId);

   /**
    * Create new button for given container and text.
    * 
    * @param   id
    *          The widget's ID.
    * @param   container
    *          Container to which new instance will belong.
    * @param   text
    *          Button text.
    *          
    * @return  new {@link Button} instance.
    */
   Button<O> createButton(WidgetId id, Container<O> container, String text);

   /**
    * Create new toggle-box for given container and text.
    * 
    * @param   id
    *          The widget's ID.
    * @param   container
    *          Container to which new instance will belong.
    * @param   text
    *          ToggleBox label text.
    *          
    * @return  new {@link ToggleBox} instance.
    */
   ToggleBox<O> createToggleBox(WidgetId id, Container<O> container, String text);

   /**
    * Create new rectangle for given container.
    * 
    * @param   id
    *          The widget's ID.
    * @param   container
    *          Container to which new instance will belong.
    *          
    * @return  new {@link Rect} instance.
    */
   Rect<O> createRectangle(WidgetId id, Container<O> container);

   /**
    * Instantiate UI-specific version of the specified widget class.
    * 
    * @param   <T>
    *          Widget abstract class type.
    * @param   clazz
    *          Widget abstract class for which UI-specific implementation should
    *          be instantiated.
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Widget configuration container.
    *          
    * @return  new UI-specific instance of the requested class. 
    */
   <T extends Widget<O>> T create(Class<T> clazz, WidgetId id, WidgetConfig cfg);

   /**
    * Create UI-specific {@link ScrollableList} implementation.
    * 
    * @param   parent
    *          Parent {@link ComboBox} widget.
    * @param   model
    *          Scrollable list model.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollableList}.
    */
   ScrollableList<O> createScrollableList(ComboBox<O> parent, ListModel<String> model);

   /**
    * Create UI-specific {@link DropDown} implementation.
    * 
    * @param   parent
    *          Parent {@link ComboBox} widget.
    *          
    * @return  instance of UI-specific implementation of {@link DropDown}.
    */
   DropDown<O> createDropDown(ComboBox<O> parent);

   /**
    * Create UI-specific {@link ScrollContainer} implementation.
    * 
    * @param   isScrollable
    *          The boolean flag that indicates whether the ScrollContainer has
    *          the scrolling capability.
    * @param   frame 
    *          Frame widget which will own the container.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollPane}.
    */
   ScrollContainer<O> createScrollContainer(boolean isScrollable, Frame<O> frame);

   /**
    * Create UI-specific {@link ScrollPane} implementation.
    * 
    * @param   id
    *          The widget's ID.
    * @param   widget 
    *          Widget which will be scrolled.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollPane}.
    */
   ScrollPane<O> createScrollPane(WidgetId id, Widget<O> widget);

   /**
    * Create UI-specific {@link Label} instance as a copy of given {@link Label}.
    * 
    * @param   id
    *          The widget's ID.
    * @param   labelInstance
    *          Source {@link Label} instance.
    * @param   frameId
    *          The ID of the frame to which the label belongs, or -1 if unknown at the time of the 
    *          creation.
    *          
    * @return  instance of UI-specific implementation of {@link Label}.
    */
   Label<O> createLabel(WidgetId id, Label<O> labelInstance, int frameId);
   
   /**
    * Create UI-specific {@link RadioButton} instance.
    * 
    * @param   config
    *          Parent config. 
    * @param   label
    *          Radio button label. 
    * @param   value
    *          Radio button data type. 
    * 
    * @return  instance of UI-specific implementation of {@link RadioButton}.
    */
   RadioButton<O> createRadioButton(RadioSetConfig config, String label, BaseDataType value);

   /**
    * Create UI-specific {@link Window} instance.
    * 
    * @param   id
    *          The widget's ID.
    * 
    * @return  instance of UI-specific implementation of {@link Window}.
    */
   Window<O> createWindow(WidgetId id);

   /**
    * Create UI-specific {@link MessageArea} instance.
    * 
    * @param   id
    *          The widget's ID.
    * @param   lines
    *          Number of visible lines in message area.         
    * 
    * @return  instance of UI-specific implementation of {@link MessageArea}.
    */
   MessageArea<O> createMessageArea(WidgetId id, int lines);
   
   /**
    * Create UI-specific {@link StatusLine} instance.
    * 
    * @param   id
    *          The widget's ID.
    * 
    * @return  instance of UI-specific implementation of {@link StatusLine}.
    */
   StatusLine<O> createStatusLine(WidgetId id);

   /**
    * Create UI-specific {@link Image} instance.
    * 
    * @param   id
    *          The widget's ID.
    * 
    * @return  instance of UI-specific implementation of {@link Image}.
    */
   Image<O> createImage(WidgetId id);
}
